//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#pragma link "PrnUtils"
#pragma link "PrnForm"
#pragma link "PrnDIB"

#include "TExcellentExampleMain.h"
#include "shellapi.h"
#include "Printers.hpp"
#include "PrnDib.h"
#include "PrnForm.h"
#include "PrnUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
}


//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
  // Load test.bmp into the Image1 component
  if (! FileExists("test.bmp")) {
    ShowMessage("Error - test.bmp not found in current directory!");
  } else {
    Image1->Picture->LoadFromFile("test.bmp");
  }
}


//---------------------------------------------------------------------------
//-- Print the form scaled to a full page - with options! -------------------
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonPrintFormClick(TObject *Sender)
{
  TPrintFormData PrintOptions;

  // Are we printing something right now?
  if (Printer()->Printing) {
    ShowMessage("Already printing... Please try again later!");
    return;
  }

  // Let the user select a printer
  if (! PrintDialog1->Execute()) {
    return;
  }

  // Prepare to set printing options
  memset(&PrintOptions, 0, sizeof(PrintOptions));
  PrintOptions.StrucSize = sizeof(PrintOptions);

  // Lets show a status dialog during the print job!
  PrintOptions.UseAbortDialog = TRUE;

  // Lets show messages if there are problems printing the form
  PrnFormSetDebugMsg(TRUE);

  // Allow the user to force the use 24 bit printing for problem printers
  PrintOptions.Use24BitOutputOnly = CheckBox24BitFormOutput->Checked;

  // Allow user to force the use of DDBs for problem printers or speed up the print job
  PrnFormSetDebugUseDDB(CheckBoxUseDDB->Checked);

  // Set print job size reduction factor
  if (CheckBoxCompress->Checked) {
    PrnFormSetOutputScaleFactor(2, 2);
  } else {
    PrnFormSetOutputScaleFactor(1, 1);
  }

  // Set WinScale
  if (CheckBoxWinStretch->Checked) {
    PrnFormSetOutputScaleFactor(1, 1);
    PrnFormSetDoWinScale(TRUE);
  } else {
    PrnFormSetDoWinScale(FALSE);
  }

  // Start the print job!
  Printer()->BeginDoc();

  // Print the form!
  if (! PrintFormEx(this,
                    Printer()->Canvas->Handle,
                    TRUE, // Center the form on the page
                    TRUE, // Adjust for perfect margins
                    &PrintOptions)) {
    // Abort the print job!
    Printer()->Abort();
    ShowMessage("User abort or other error!");
  } else {
    // End the print job!
    Printer()->EndDoc();
  }

  // We are done!
  ShowMessage("Printing Completed!");

}



//---------------------------------------------------------------------------
//-- A callback function for updating an image printer abort dialog! --------
//---------------------------------------------------------------------------
BOOL __stdcall ImagePrinterCallbackFunction(DWORD UnitsDone,
                                            DWORD TotalUnits,
                                            TAbortDialog * AbortDialog)
{                                     
  System::AnsiString s;

  // Update the abort dialogs caption to give a status update
  s = "Printing  - " +
      IntToStr(INT((((float)UnitsDone / (float)TotalUnits) * 100))) +
      "%";
  AbortDialogSetCaption(AbortDialog,
                        s.c_str());
  // Return if the user has canceled the print job.
  return(! AbortDialogUserHasCanceled(AbortDialog));
}


//---------------------------------------------------------------------------
//-- Print test.bmp scaled to a full page - with abort dialog! --------------
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonPrintImageClick(TObject *Sender)
{
  TAbortDialog *AbortDialog;  // The abort dialog
  BITMAPINFO * BitmapInfo;    // A pointer to a BitmapInfo structure
  void * Bits;                // A pointer to the bitmap bits
  int BitmapWidth;            // The bitmap width
  int BitmapHeight;           // The bitmap height

  // Are we printing something right now?
  if (Printer()->Printing) {
    ShowMessage("Already printing... Please try again later!");
    return;
  }

  // Do we have a bitmap to print?
  if (! FileExists("test.bmp")) {
    ShowMessage("Error - test.bmp not found in current directory!");
    return;
  }

  // Let the user select a printer
  if (! PrintDialog1->Execute()) {
    return;
  }

  // Load test.bmp as a Device Independent Bitmap
  if (! LoadDIBFromFile("test.bmp",
                        (PVOID*)&BitmapInfo,
                        &Bits,
                        &BitmapWidth,
                        &BitmapHeight)){
    ShowMessage("Bitmap load error");
    return;
  }

  // Create the abort dialog
  AbortDialog = CreateAbortDialog(Application->Handle, this);
  PrnDIBSetAbortDialogHandle(AbortDialog->Handle);

  // Lets show messages if there are problems printing the form
  PrnDIBSetDebugMsg(TRUE);

  // Allow user to force the use of DDBs for problem printers or speed up the print job
  PrnDIBSetDebugUseDDB(CheckBoxUseDDB->Checked);

  // Set print job size reduction factor
  if (CheckBoxCompress->Checked) {
    PrnDIBSetOutputScaleFactor(2, 2);
  } else {
    PrnDIBSetOutputScaleFactor(1, 1);
  }

  // Set WinScale
  if (CheckBoxWinStretch->Checked) {
    PrnDIBSetDoWinScale(TRUE);
    PrnDIBSetOutputScaleFactor(1, 1);
  } else {
    PrnDIBSetDoWinScale(FALSE);
  }

  // Start the print job!
  Printer()->BeginDoc();

  // Print the image!
  if (PrintDIBitmapCB(Printer()->Canvas->Handle,
                      BitmapInfo,
                      Bits,
                      TRUE,
                      TRUE,
                      (TAppCallbackFn)ImagePrinterCallbackFunction,
                      DWORD(AbortDialog)) < PRINT_SUCCESSFUL) {
    // Abort the print job!
    Printer()->Abort();
    ShowMessage("User abort or other error!");
  } else {
    // End the print job!
    Printer()->EndDoc();
  }

  // Clean Up!
  FreeMemEx(BitmapInfo);
  FreeMemEx(Bits);
  FreeAbortDialog(AbortDialog);
  PrnDIBSetAbortDialogHandle(0);

  // We are done!
  ShowMessage("Printing Completed!");

}


//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonGetDocumentationClick(TObject *Sender)
{
  // Open web browser and go to the TExcellent documentation page
  ShellExecute(this->Handle,
               NULL,
               "http://www.code4sale.com/joehecht/TExcellentDocs.htm",
               NULL,
               NULL,
               SW_SHOWNORMAL);
}


//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonBuyFormPrinterClick(TObject *Sender)
{
  // Open web browser and go to the TExcellentFormPrinter BuyIt! page
  ShellExecute(this->Handle,
               NULL,
               "http://www.code4sale.com/order/texcellentformprinter.htm",
               NULL,
               NULL,
               SW_SHOWNORMAL);
}


//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonBuyImagePrinterClick(TObject *Sender)
{
  // Open web browser and go to the TExcellentImagePrinter BuyIt! page
  ShellExecute(this->Handle,
               NULL,
               "http://www.code4sale.com/order/texcellentimageprinter.htm",
               NULL,
               NULL,
               SW_SHOWNORMAL);
}


//---------------------------------------------------------------------------
void __fastcall TForm1::LabelMailToClick(TObject *Sender)
{
  // Open the email application and send mail to sales@code4sale.com
  ShellExecute(this->Handle,
               NULL,
               "mailto:sales@code4sale.com",
               NULL,
               NULL,
               SW_SHOWNORMAL);
}


//---------------------------------------------------------------------------
void __fastcall TForm1::LabelWebHomePageClick(TObject *Sender)
{
  // Open web browser and go to Joe Hecht Associates web page
  ShellExecute(this->Handle,
               NULL,
               "http://www.code4sale.com/joehecht/index.htm",
               NULL,
               NULL,
               SW_SHOWNORMAL);
}


//---------------------------------------------------------------------------
void __fastcall TForm1::LabelTryOfficialDemoApplicationClick(
      TObject *Sender)
{
  // Open TExcellentDemo program or try to download it
  if (FileExists("TExcellentDemo.exe")) {
    WinExec("TExcellentDemo.exe",
            SW_SHOWNORMAL);
  } else {
    ShellExecute(this->Handle,
                 NULL,
                 "http://www.code4sale.com/joehecht/TExcellentDemo.zip",
                 NULL,
                 NULL,
                 SW_SHOWNORMAL);
  }
}

//---------------------------------------------------------------------------

